/*
	ATtiny2313, 8 MHz

	PB.0 >> out "-" GREEN_1
	PB.1 >> out "-" YELLOW_1
	PB.2 >> out "-" RED_1
	PB.4 >> (PCINT4) in from TSOP

	PD.2 >> (INT0) in SW
	PD.3 >> out "-" RED_2
	PD.4 >> out "-" YELLOW_2
	PD.5 >> out "-" GREEN_2
	
*/

.include "tn2313def.inc"
.include "IR_Svetofor_macros.inc"

//----------- SRAM
.dseg
buffer:	.byte(32)		;     32      

//----------- FLASH
.cseg

//-----------  
	rjmp RESET		;Reset Handler
	rjmp INT_0		;External Interrupt0 Handler
	reti					;rjmp INT1 ; External Interrupt1 Handler
	reti					;rjmp TIM1_CAPT ; Timer1 Capture Handler
	rjmp TIM1_COMPA	;Timer1 CompareA Handler
	reti					;rjmp TIM1_OVF ; Timer1 Overflow Handler
	rjmp TIM0_OVF		;Timer0 Overflow Handler
	reti					;rjmp USART0_RXC ; USART0 RX Complete Handler
	reti					;rjmp USART0_DRE ; USART0,UDR Empty Handler
	reti					;rjmp USART0_TXC ; USART0 TX Complete Handler
	reti					;rjmp ANA_COMP ; Analog Comparator Handler
	rjmp PCINT_4			;Pin Change Interrupt
	reti					;rjmp TIMER1_COMPB ; Timer1 Compare B Handler
	reti					;rjmp TIMER0_COMPA ; Timer0 Compare A Handler
	reti					;rjmp TIMER0_COMPB ; Timer0 Compare B Handler
	reti					;rjmp USI_START ; USI Start Handler
	reti					;rjmp USI_OVERFLOW ; USI Overflow Handler
	reti					;rjmp EE_READY ; EEPROM Ready Handler
	reti					;rjmp WDT_OVERFLOW ; Watchdog Overflow Handler

//-----------  
RESET: 
		//Set Stack Pointer to top of RAM
		ldi temp, low(RAMEND)
		out SPL,temp

		//   , F_CPU = 8 MHz
		ldi temp,0x80
		out CLKPR,temp
		clr temp
		out CLKPR,temp

		//PORTB
		ldi temp,0b00000011
		out PORTB,temp
		ldi temp,0b00000111
		out DDRB,temp

		//PORTD
		ldi temp,0b00110100
		out PORTD,temp
		ldi temp,0b00111000
		out DDRD,temp

		//      INT_0
		ldi temp,(1<<ISC01)
		out MCUCR,temp		;The falling edge of INT0 generates an interrupt request
		//      PB.4
		ldi temp,(1<<PCINT4)
		out PCMSK,temp		;Pin_4 Change Enable Mask
		ldi temp,(1<<INT0)|(1<<PCIE)
		out GIMSK,temp		;External Interrupt Request_1 and Pin_4 Change Interrupt Enable

		// Timer/Counter1
		clr temp
		out TCCR1A,temp
		ldi temp,(1<<WGM12)|(1<<CS11)|(1<<CS10)
		out TCCR1B,temp	;CTC, clk/64 (From prescaler)
		ldi temp,high(55860)
		out OCR1AH,temp
		ldi temp,low(55860)
		out OCR1AL,temp	;  0,5 

		// Timer/Counter0
		clr temp
		out TCCR0A,temp
		ldi temp,(1<<CS02)|(1<<CS00)
		out TCCR0B,temp	;Normal, clk/1024 (From prescaler)

		//Timer/Counter1 >> Output Compare A Match Interrupt Enable
		//Timer/Counter0 >> Overflow Interrupt Enable
		ldi temp,(1<<OCIE1A)|(1<<TOIE0)
		out TIMSK,temp

		//UART
		clr temp
		out UCSRA,temp
		ldi temp,(1<<TXEN)
		out UCSRB,temp		;Transmitter Enable
		ldi temp,(1<<UCSZ1)|(1<<UCSZ0)
		out UCSRC,temp		;8n1
		ldi temp,high(51)
		out UBRRH,temp
		ldi temp,low(51)
		out UBRRL,temp	;9600

		clr traffic_counter
		mov null,traffic_counter
		ldi temp,(1<<0)
		mov change_green_B,temp
		ldi temp,(1<<5)
		mov change_green_D,temp

		sei		;Enable interrupts

//----------- Main program start
main:
		rjmp main

//------------------------------------------------------------
//-----------     ----------- 
//------------------------------------------------------------

//-----------    
INT_0:
		push temp

		rcall pause	;
				
		sbis PIND,2	;   -      , ...
		rjmp PC+2		;   -  ,   ,  
		rjmp out_INT_0	;...   

		sbis PINB,1
		rjmp out_INT_0	;   -  ,   )))

		//  traffic_counter     
		sbic PINB,2	;  , ...
		rjmp PC+3
		ldi traffic_counter,(cycle+5)	;...   
		rjmp out_INT_0
		cpi traffic_counter,(cycle-1)		;   ...
		brsh PC+2						;...     
		ldi traffic_counter,(cycle-1)		; -  

	out_INT_0:	
		ldi temp,(1<<INTF0)
		out EIFR,temp	;External Interrupt Flag 0 is Cleared
		ldi bit_counter,0xFE	;   -

		pop temp
		reti

//-----------   70 mS
pause:
		ldi r24,0x80
		ldi r25,0xB5
		ldi r23,0x01
		sbiw r24,1
		sbc r23,null
		brcc PC-2
		ret

//-----------   0,5  -      
TIM1_COMPA:
		inc traffic_counter
		
		cpi traffic_counter,cycle	;    / ?
		brlo out_TIM1_COMPA			; -      

		cpi traffic_counter,(cycle+6)		;    ...
		brlo prepare_for_change			;...      

		cpi traffic_counter,(cycle+12)	;   ...
		brlo lights_yellow				;...   

		cpi traffic_counter,(cycle+13)	;   ...
		brlo change_main_lights			;...      

		//     
	prepare_for_change:
		sbis PINB,2				;   ...
		rjmp out_TIM1_COMPA			;...      
		in temp,PINB				;   PORTB...
		eor temp,change_green_B		;...   ...
		out PORTB,temp				;...   -      PORTB
		in temp,PIND				;  PORTD    ...
		eor temp,change_green_D		;
		out PORTD,temp				;
		sbi PORTD,2				;...        pull-up  INT_0
		rjmp out_TIM1_COMPA			

		// 
	lights_yellow:
		gr1_off
		gr2_off
		yl1_on
		yl2_on
		rjmp out_TIM1_COMPA

		//        
	change_main_lights:
		sbis PINB,2	;    - 
		rjmp PC+8		;  - 
		rd1_on
		rd2_on
		yl1_off
		yl2_off
		gr1_off
		gr2_off
		rjmp PC+7
		rd1_off
		rd2_off
		yl1_off
		yl2_off
		gr1_on
		gr2_on
		clr traffic_counter
		
	out_TIM1_COMPA:
		ldi temp,(1<<INTF0)
		out EIFR,temp			;External Interrupt Flag 0 is Cleared
		reti

//-----------    33 mS
TIM0_OVF:
		ldi bit_counter,0xFE	;   -
		reti

//-----------     TSOP
PCINT_4:
		sbic PINB,4	;     
		reti			;    - 

		//      ...
		in temp,TCNT0	;  
		out TCNT0,null	; 

		cpi bit_counter,0xFE	;  254 (254 -    ,    -   )
		brlo bit_define		; ,  -   -    
		cpi bit_counter,0xFF	;  255 -     ,  -  
		brlo PC+6				;    -    .    -
		cpi temp,80	;90 -  11.5 mS, - > 13.5 mS
		brsh PC+3		;  -      -,       
		ldi bit_counter,0xFE	;  -  ,     ...
		reti					;...    
		//    
		ldi YH,high(buffer)
		ldi YL,low(buffer)	;    Y         
		inc bit_counter	; bit_counter...
		reti				;...    

	bit_define:
		cpi temp,13	;1.13 mS (<9 ) = 0; 2.26 mS (>17 ) = 1; 13 -  
		brlo PC+3		;     ...
		ldi temp,1	; -   
		rjmp PC+2		;       
		ldi temp,0	;... 
		st Y+,temp	;      
		inc bit_counter	; 
		cpi bit_counter,32	; -     ?
		breq compare_code	;  -       
		reti				;  -        

	compare_code:
		//  UART  
		rcall UART_out

		//       FLASH
		//    REFRESH  FOLDER_PLUS -       Z
		ldi YH,high(buffer)
		ldi YL,low(buffer)		;    Y         
		ldi ZH,high(SW_Fpl*2)
		ldi ZL,low(SW_Fpl*2)	;    Z      

		ldi count,32	;  32 
	next_comp_REFR:	
		ld bit_counter,Y+
		lpm temp,Z+
		cp temp,bit_counter
		brne set_to_zero
		dec count
		brne next_comp_REFR
		
		//   -      (  )
		sbis PINB,1
		rjmp set_to_zero	;   -  ,   )))
		sbic PINB,2		;  , ...
		rjmp PC+3
		ldi traffic_counter,(cycle+5)	;...   
		rjmp set_to_zero
		cpi traffic_counter,(cycle-1)		;  ...
		brsh PC+2						;...     
		ldi traffic_counter,(cycle-1)		;...  
		
	set_to_zero:
		ldi bit_counter,0xFE	;       -
		ldi temp,(1<<INTF0)
		out EIFR,temp	;External Interrupt Flag 0 is Cleared
		reti

//-----------   UART   SRAM    
UART_out:
		ldi YH,high(buffer)
		ldi YL,low(buffer)	;    Y         
	
		ldi count,32	;  32 
	next_UART_bit:	
		ld bit_counter,Y+
		sbis UCSRA,UDRE	;Wait for empty transmit buffer
		rjmp PC-1
		out UDR,bit_counter	;Put data into buffer, sends the data
		dec count
		brne next_UART_bit

		ret

//-----------   
SW_REF:	.db 0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,0,0,0,0,1,1,1,0,1	;    
SW_Fpl:	.db 0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,1,0,0,0,1,0,0,1,0,1,1,1,0,1	;    
